@@ -33,9 +33,9 @@
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
-#define GS_SERVICE   "org.gnome.screensaver"
-#define GS_PATH      "/org/gnome/screensaver"
-#define GS_INTERFACE "org.gnome.screensaver"
+#define GS_SERVICE   "org.gnome.ScreenSaver"
+#define GS_PATH      "/org/gnome/ScreenSaver"
+#define GS_INTERFACE "org.gnome.ScreenSaver"
 
 /* this is for dbus < 0.3 */
 #if ((DBUS_VERSION_MAJOR == 0) && (DBUS_VERSION_MINOR < 30))
@@ -183,7 +183,7 @@ do_command (DBusConnection *connection)
         DBusMessage *reply;
 
         if (do_quit) {
-                reply = screensaver_send_message_void (connection, "quit", FALSE);
+                reply = screensaver_send_message_void (connection, "Quit", FALSE);
                 goto done;
         }
 
@@ -205,15 +205,15 @@ do_command (DBusConnection *connection)
         }
 
         if (do_lock) {
-                reply = screensaver_send_message_void (connection, "lock", FALSE);
+                reply = screensaver_send_message_void (connection, "Lock", FALSE);
         }
 
         if (do_cycle) {
-                reply = screensaver_send_message_void (connection, "cycle", FALSE);
+                reply = screensaver_send_message_void (connection, "Cycle", FALSE);
         }
 
         if (do_poke) {
-                reply = screensaver_send_message_void (connection, "poke", FALSE);
+                reply = screensaver_send_message_void (connection, "Poke", FALSE);
         }
 
         if (do_activate) {
@@ -284,7 +284,6 @@ main (int    argc,
                 g_warning ("Locale not understood by C library, internationalization will not work\n");
 
         context = g_option_context_new (NULL);
-        g_option_context_set_ignore_unknown_options (context, TRUE);
         g_option_context_add_main_entries (context, entries, NULL);
         retval = g_option_context_parse (context, &argc, &argv, &error);
 
